/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <assert.h>

// define some constants to make referencing in/outs clearer
#define PN_INPUT1 0
#define PN_INPUT2 1
#define PN_OUTPUT1 2
#define PN_BOOL 3
static float mem;

// The most important part, processing the audio
void Module::sub_process(long buffer_offset, long sampleFrames )
{
	// assign some pointers to your in/output buffers. usually blocks (array) of 96 samples
    float *in1  = buffer_offset + input1_buffer;
    float *in2  = buffer_offset + input2_buffer;
    float *out1 = buffer_offset + output1_buffer;

    while(--sampleFrames >= 0) // sampleFrames = how many samples to process (can vary). repeat (loop) that many times
    {
		// do the actual processing 
		
		if (*in1 > 0.2) mem = 1.0;
		if (*in2 > 0.2) mem = 0.0;
		if ((mybool == true ) && (*in1 > 0.2 )) mem = 1.0;
         

		*out1 = mem;	// store the result in the output buffer

		// increment the pointers (move to next sample in buffers)
		in1++;
		in2++;
		out1++;
    }
}

Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

void Module::open()
{
	SEModule_base::open();
	mem = 0.0;

	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);
}

Module::~Module()
{
	// This is where you free any memory/resources your module has created
}

// describe your module
bool Module::getModuleProperties (SEModuleProperties* properties)
{
	// describe the plugin, this is the name the end-user will see.
	properties->name = "FlipFlop1";								// !!TODO!!

	// return a unique string 32 characters max
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug.
	// No two plugs may have the same id.
	properties->id = "ralphs FF1";						// !!TODO!!

	// Info, may include Author, Web page whatever
#if defined(MINGW) || defined(__GNUC__)
	properties->about = "by Jeff M (GNU)";								// !!TODO!!
#elif defined _MSC_VER
	properties->about = "by ralph a. cronin \n  FAKE MUSIC \n ( searching jazz bass to my piano 2009 ) \n and by Jeff Mc Clintock" ;
#else
	properties->about = "by Jeff M (Borland?)";
#endif
	return true;
}

// describe the pins (plugs)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )								// !!TODO!! list your in / out plugs
	{
		// typical input plug (inputs are listed first)
	case 0:
		properties->name				= "set";
		properties->variable_address	= &input1_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
	case 1:
		properties->name				= "reset";
		properties->variable_address	= &input2_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
		// typical output plug
	case 2:
		properties->name				= "Q";
		properties->variable_address	= &output1_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
    case 3:
		properties->name				= "SET-PRIORITY";
		properties->variable_address	= &mybool;
		properties->direction			= DR_IN;
		properties->datatype			= DT_BOOL;
		break;
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}

// An input plug has changed value
void Module::OnPlugStateChange(SEPin *pin)
{
	/*
	// query the 'state of the input plugs...
	//     ST_RUN     = Normal Streaming Audio        (e.g. from an oscillator)
	//     ST_STATIC  = Fixed, unchanging input value (e.g. a slider at rest)
	state_type in_stat1 = getPin(PN_INPUT1)->getStatus();
	state_type in_stat2 = getPin(PN_INPUT2)->getStatus();

	// we need to pass on the state of this module's output signal
	// it depends on the inputs...
	state_type out_stat = in_stat1;
	if( in_stat2 > out_stat)
		out_stat = in_stat2;

	// if either input zero, tell downstream modules audio has stopped
	if( in_stat1 < ST_RUN && getPin(PN_INPUT1)->getValue() == 0.f )
	{
		out_stat = ST_STATIC;
	}

	if( in_stat2 < ST_RUN && getPin(PN_INPUT2)->getValue() == 0.f )
	{
		out_stat = ST_STATIC;
	}
    */

	// 'transmit' new output status to next module 'downstream'
	getPin(PN_OUTPUT1)->TransmitStatusChange( SampleClock(), ST_RUN);// statt out_stat ralph
}

